/*
    Copyright (C) 1999 Paul Barton-Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: ardour_ui.h,v 1.42 2000/11/04 04:53:33 pbd Exp $
*/

#ifndef __ardour_gui_h__
#define __ardour_gui_h__

#include <pbd/touchable.h>

#include <gtkmmext/gtk_ui.h>
#include <gtkmmext/pix.h>
#include <gtkmmext/spinner.h>
#include <gtkmmext/levelhold.h>
#include <gtkmmext/pixmap_button.h>
#include <gtkmmext/popup_selector.h>
#include <gtkmmext/click_box.h>
#include <gtkmmext/choice.h>

#include "tape.h"

class Track_UI;
class AudioClock_UI;
class Editor;
class Location_UI;
class LocationStack;

namespace ALSA {
	class MultiChannelDevice;
};

namespace MIDI {
	class MachineControl;
};

namespace Gtkmmext {
	class LED;
};

class ARDOUR_UI : public Gtkmmext::UI

{
  public:
	ARDOUR_UI (ALSA::MultiChannelDevice *, 
		   MIDI::MachineControl *,
		   int *, char ***, const char *);
	~ARDOUR_UI();

	Gtkmmext::Pix *rec_enable_button_pix ();
	Gtkmmext::Pix *monitor_button_pix ();
	Gtkmmext::Pix *push_button_pix ();
	Gtkmmext::Pix *small_push_button_pix ();
	Gtkmmext::Pix *toggle_button_pix ();

	static ARDOUR_UI *instance () { return theHdrUI; }

	int load_tape (const string &name, const string &path);

	float preroll () { return preroll_adjustment.get_value(); }
	float postroll () { return postroll_adjustment.get_value(); }
	
	void switch_to_notebook_page (int);
	void toggle_control_window ();
	void toggle_location_window ();

  protected:
	friend class Editor;

	/* Button panel functions, also accessed via
	   key strokes from the editor.
	*/

	void toggle_metering ();
	void toggle_clock ();
	void toggle_auto_play ();
	void toggle_auto_input ();
	void toggle_auto_loop ();
	void toggle_auto_punch ();
	void toggle_dynamic_punch ();
	void toggle_rehearse ();
	void toggle_single_button_record ();
	void toggle_all_safe ();
	void toggle_monitor_in ();
	void toggle_bypass ();
	void toggle_hw_monitoring ();

  protected:
	friend class Location_UI;
	void transport_locate (size_t position);

  private:
	struct GlobalClickBox : public Gtk::VBox {
	    Gtkmmext::ClickBox  *box;
	    Gtk::Frame      frame;
	    Gtk::Label      label;
	    vector<string> &strings;
	    Gtk::Adjustment adjustment;

	    static void printer (char buf[32], Gtk::Adjustment &adj, void *arg);

	    GlobalClickBox (const string &str, vector<string> &vs)
		    : strings (vs),
		      adjustment (0, 0, vs.size() - 1, 1, 1, 0) {
		    box = new Gtkmmext::ClickBox (&adjustment, "ClickButton");
		    label.set_text (str);
		    label.set_name ("GlobalButtonLabel");
		    frame.add (*box);
		    frame.set_shadow_type (GTK_SHADOW_IN);
		    pack_start (label);
		    pack_start (frame);
		    box->set_print_func (printer, this);
		    box->set_wrap (true);
	    };
	};

	ALSA::MultiChannelDevice *mcd;
	MIDI::MachineControl     *mmc;
	ARDOUR::Tape            *tape;

	bool                    metering;
	bool                   _input_metering;
	gint                    meter_timer_tag;

	/* for the main window */

	Gtk::Window              main_window;
	Gtk::VBox                main_window_vpacker;
	Gtk::Notebook            notebook;
	Gtk::Fixed               recorder_base;
	Gtk::HBox                editor_base;
	Gtk::VBox                recorder_vpacker;
	Gtk::HBox                upper_hpacker;

//	Gtk::HBox                middle_hpacker;
//	Gtk::HBox                lower_hpacker;

	/* for the transport_clock area*/

	Gtk::Fixed  tc_base;
	Gtk::HBox   tc_hpacker;
	Gtk::VBox   tc_vpacker;
	/* for the location window */

	/* Location interface */

	Gtk::Frame              location_frame;
	Location_UI            *location_ui;
	LocationStack          *locations;

	Gtk::Window location_window;
	Gtk::Fixed  location_base;
	Gtk::VBox   location_vpacker;
	Gtk::HBox   location_hpacker;

	/* for the controls window */

	Gtk::Window control_window;
	Gtk::Fixed  ctl_base;
	Gtk::HBox   ctl_hpacker;
	Gtk::VBox   ctl_vpacker;

	Gtkmmext::Pix          *rec_enable_buttons;
	Gtkmmext::Pix          *monitor_buttons;
	Gtkmmext::Pix          *push_buttons;
	Gtkmmext::Pix          *small_push_buttons;
	Gtkmmext::Pix          *toggle_buttons;

	/* Logo */

	Gtk::HBox                logo_hpacker;
	Gtk::VBox                logo_vpacker;
	Gtk::Label               logo_label1;
	Gtk::Label               logo_label2;
	Gtk::Pixmap             *penguin_pixmap;
	Gtk::HSeparator          hsep1;
	Gtk::HBox                hsep1_box;
	Gtk::HSeparator          hsep2;
	Gtk::HBox                hsep2_box;

	/* track display */

	Gtk::VBox                track_vbox;
	Gtk::Fixed               upper_track_box;
	Gtk::Fixed               lower_track_box;
	Gtk::Fixed               meter_base;
	Gtk::Frame               meter_frame;
	Track_UI               **track_ui;

	Gtk::Entry               tape_id_display;
	Gtk::VBox                tape_id_vbox;
	Gtk::Frame               tape_id_frame;

	GlobalClickBox     *sample_format_button;
	vector<string>      sample_format_strings;

	GlobalClickBox    *rate_button;
	vector<string>     rate_strings;

	Gtk::HBox                clock_and_button_hbox;
	Gtk::HBox                transport_and_tape_hbox;

	Gtk::Table               button_table;
	Gtk::HBox                button_hbox;
	Gtk::VBox                button_vbox;
	Gtk::Frame               button_frame;
	Gtk::Fixed               button_base;

	Gtk::Table               adjuster_table;
	Gtk::Frame               adjuster_frame;
	Gtk::Fixed               adjuster_base;

	Gtkmmext::PixmapButton  *clock_button;
	Gtk::Label               clock_button_label;
	Gtkmmext::PixmapButton  *global_meter_button;
	Gtk::Label          global_meter_button_label;
	Gtkmmext::PixmapButton  *reset_meters_button;
	Gtk::Label          reset_meters_button_label;

	Gtkmmext::PixmapButton  *dynamic_punch_button;
	Gtk::Label          dynamic_punch_button_label;
	Gtkmmext::PixmapButton  *auto_punch_button;
	Gtk::Label          auto_punch_button_label;
	Gtkmmext::PixmapButton  *auto_loop_button;
	Gtk::Label          auto_loop_button_label;

	Gtkmmext::PixmapButton  *auto_play_button;
	Gtk::Label          auto_play_button_label;
	Gtkmmext::PixmapButton  *auto_input_button;
	Gtk::Label          auto_input_button_label;
	Gtkmmext::PixmapButton  *bypass_button;
	Gtk::Label          bypass_button_label;

	Gtkmmext::PixmapButton  *rehearse_button;
	Gtk::Label          rehearse_button_label;
	Gtkmmext::PixmapButton  *all_safe_button;
	Gtk::Label          all_safe_button_label;
	Gtkmmext::PixmapButton  *single_button_record_button;
	Gtk::Label               single_button_record_button_label;

	Gtkmmext::PixmapButton  *hw_monitor_button;
	Gtk::Label               hw_monitor_button_label;

	Gtkmmext::PixmapButton  *save_button;
	Gtk::Label save_button_label;

	Gtkmmext::PixmapButton  *restore_button;
	Gtk::Label restore_button_label;

	GlobalClickBox     *online_control_button;
	vector<string>      online_control_strings;

	GlobalClickBox    *crossfade_time_button;
	vector<string>     crossfade_time_strings;

	GlobalClickBox    *mmc_id_button;
	vector<string>     mmc_id_strings;

	GlobalClickBox    *sample_clock_mode_button;
	vector<string>     sample_clock_mode_strings;

	Gtk::Adjustment     xfade_adjustment;
	Gtk::Adjustment     preroll_adjustment;
	Gtk::Adjustment     postroll_adjustment;
	Gtkmmext::ClickBox *xfade_button;
	Gtk::Frame          xfade_frame;
	Gtk::Label          xfade_label;
	Gtk::VBox           xfade_vbox;
	Gtkmmext::ClickBox *preroll_button;
	Gtk::Frame          preroll_frame;
	Gtk::Label          preroll_label;
	Gtk::VBox           preroll_vbox;
	Gtkmmext::ClickBox *postroll_button;
	Gtk::Frame          postroll_frame;
	Gtk::Label          postroll_label;
	Gtk::VBox           postroll_vbox;

	int  build_track_uis ();
	int  setup_pixmaps ();
	int  setup_windows (int argc, char **argv);
	void setup_tape_menu ();
	void setup_transport ();
	void setup_clock ();
	void setup_logo ();
	void setup_tape_info ();
	void setup_adjustables ();
	void setup_button_table ();
	
	Gtk::MenuBar* make_menubar ();
	
	static ARDOUR_UI *theHdrUI;
	static gint update_tracks (void *);

	void startup ();
	void shutdown ();

	void channel_status_redisplay (size_t chn, char status);

	void start_metering ();
	void stop_metering ();


	void record_status_change ();
	void transport_change ();

	void map_some_tape_state (Gtkmmext::PixmapButton *button,
				  bool (ARDOUR::Tape::*get)());

	void toggle_some_tape_state (Gtkmmext::PixmapButton *button,
				     bool (ARDOUR::Tape::*get)(),
				     void (ARDOUR::Tape::*set)(bool));
	void map_button_state ();

	void clear_meters ();
	void reset_meters ();


	static gint _blink_handler (void *);
	void blink_handler ();
	gint blink_handler_tag;
	bool rec_enable_blink_on;

	void set_xfade ();

	static int _load_default_tape (void *);
	void load_default_tape ();

	void sample_clock_mode_adjusted ();
	void control_methods_adjusted ();
	void mmc_device_id_adjusted ();

	guint last_notebook_page;
	void notebook_switched (Gtk::Notebook_Helpers::Page *, guint);
	void switched_to_recorder ();
	void switched_to_editor ();
	void switched_to_twotrack ();

	void map_tape_description ();

	void waiting();
	void done_waiting ();
	void display_transport_state ();
	void display_record_state ();

  protected:
	struct FakeTransportDisplay;
	friend struct FakeTransportDisplay;
	void map_transport_state ();

	struct FakeRecordDisplay;
	friend struct FakeRecordDisplay;
	void map_record_state ();

	/* These are strange objects. We use them so that we can send
	   a GTK_UI "TouchDisplay" message whenever some change occurs
	   asychronously. Since that message is generated (sometimes)
	   by another thread, it can't (always) be processed in the
	   same thread as its generated in. In such cases, we send a
	   TouchDisplay message, which will cause this object's
	   "touch" method to be called within the GUI thread, which
	   will then do the right thing.
	*/

	struct FakeTransportDisplay : public Touchable {
	    ARDOUR_UI *ui;
	    void touch () {
		    ui->map_transport_state ();
	    }
	};
	  
	FakeTransportDisplay fakeTransportDisplay;

	struct FakeRecordDisplay : public Touchable {
	    ARDOUR_UI *ui;
	    void touch () {
		    ui->map_record_state ();
	    }
	};

	FakeRecordDisplay fakeRecordDisplay;

	struct FakeHalt : public Touchable {
	    ARDOUR_UI *ui;
	    void touch () {
		    ui->notify_mcd_halted ();
	    }
	};

	FakeHalt fakeHalt;
	friend struct FakeHalt;
	void mcd_halted ();
	void notify_mcd_halted ();
	
  protected:
	Gtk::Frame               clock_frame;
	Gtk::Fixed               clock_base;
	Gtk::VBox                clock_packer;
	AudioClock_UI           *clock_ui;

	Gtkmmext::LED           *rec_led;
	Gtk::Pixmap             *rec_pixmap;
	Gtk::EventBox            rec_button;
	Gtk::HBox                rec_hbox;
	Gtk::VBox                rec_vbox;
	Gtk::Label               rec_label;

	Gtkmmext::LED           *stop_led;
	Gtk::Pixmap             *stop_pixmap;
	Gtk::EventBox            stop_button;
	Gtk::HBox                stop_hbox;
	Gtk::VBox                stop_vbox;
	Gtk::Label               stop_label;

	Gtkmmext::LED           *roll_led;
	Gtk::Pixmap             *roll_pixmap;
	Gtk::EventBox            roll_button;
	Gtk::HBox                roll_hbox;
	Gtk::VBox                roll_vbox;
	Gtk::Label               roll_label;

	Gtkmmext::LED           *forward_led;
	Gtk::Pixmap             *forward_pixmap;
	Gtk::EventBox            forward_button;
	Gtk::HBox                forward_hbox;
	Gtk::VBox                forward_vbox;
	Gtk::Label               forward_label;

	Gtkmmext::LED           *rewind_led;
	Gtk::Pixmap             *rewind_pixmap;
	Gtk::EventBox            rewind_button;
	Gtk::HBox                rewind_hbox;
	Gtk::VBox                rewind_vbox;
	Gtk::Label               rewind_label;

	Gtkmmext::LED           *tape_led;
	Gtk::Pixmap             *tape_pixmap;
	Gtk::EventBox            tape_button;
	Gtk::HBox                tape_hbox;
	Gtk::VBox                tape_vbox;
	Gtk::Label               tape_label;

	Gtk::HBox                transport_packer;
	Gtk::VBox                transport_vpacker;
	Gtk::Fixed               transport_base;
	Gtk::Frame               transport_frame;

	Gtk::Window              new_tape_dialog;

	/* These change where we accept control from:
	   MMC, X (local) or both.
	*/

	void allow_mmc_only ();
	void allow_mmc_and_local ();
	void allow_local_only ();

	/* Dialog to create a new transport */

	Gtk::Frame               new_tape_frame;
	Gtk::Entry               new_tape_name;
	Gtk::Adjustment          new_tape_length_adjustment;
	Gtk::SpinButton          new_tape_length;
	Gtk::Adjustment          new_tape_tracks_adjustment;
	Gtk::SpinButton          new_tape_tracks;
	Gtk::Label               new_tape_tracks_label;
	Gtk::Label               new_tape_name_label;
	Gtk::Label               new_tape_length_label;
	Gtk::HBox                new_tape_tracks_hbox;
	Gtk::HBox                new_tape_name_hbox;
	Gtk::HBox                new_tape_length_hbox;
	Gtk::VBox                new_tape_vpacker;
	Gtk::Button              new_tape_ok;
	Gtk::Button              new_tape_cancel;
	Gtk::HBox                new_tape_button_hbox;

	Gtk::Menu* tape_popup_menu;
	Gtkmmext::PopupSelector *tape_selector_popup;

	/* menu bar */

	Gtk::MenuBar menu_bar;
	void build_menu_bar ();
	
	void tape_add_track ();
	void tape_extend ();

	Gtk::Window           wipe_window;
	Gtk::Frame            wipe_frame;
	Gtk::VBox             wipe_box;
	Gtk::HBox             wipe_cancel_box;
	Gtk::HBox             wipe_commit_box;
	Gtkmmext::PixmapButton *wipe_commit_button;
	Gtkmmext::PixmapButton *wipe_cancel_button;
	Gtk::Label            wipe_commit_label;
	Gtk::Label            wipe_cancel_label;
	int                   wipe_level;

	int setup_wipe_window ();
	void wipe_tape ();
	void wipe_cancel ();
	int wipe_delete (GdkEventAny *);

	gint new_tape_window_deleted (GdkEventAny *);
	void tape_selected (Gtkmmext::PopupSelector *popup, 
			    Gtkmmext::SelectionResult *result);

	gint tape_menu (GdkEventButton *);
	void new_tape ();
	void cancel_new_tape ();
	void accept_new_tape ();
	void open_tape ();
	void unload_tape ();

	gint transport_stop (GdkEventButton *);
	gint transport_record (GdkEventButton *);
	gint transport_roll (GdkEventButton *);
	gint transport_forward (GdkEventButton *);
	gint transport_rewind (GdkEventButton *);
	
	void transport_locating ();
	void transport_rolling ();
	void transport_rewinding ();
	void transport_forwarding ();
	void transport_stopped ();

	void connect_to_tape (ARDOUR::Tape *);

	guint32  last_key_press_time;
	gint recorder_key_press (GdkEventKey *);
	gint recorder_key_release (GdkEventKey *);
	gint global_key_press_handler (GdkEventKey *);
	gint global_key_release_handler (GdkEventKey *);

	void save_state ();
	void restore_state ();

	void map_control_change (ARDOUR::Tape::ControlType);
	
	Editor *editor;
	int create_editor ();
};

#endif /* __ardour_gui_h__ */
