/*
    Copyright (C) 2000 Paul Barton-Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: editor.h,v 1.12 2000/11/04 16:35:47 pbd Exp $
*/

#ifndef __ardour_editor_h__
#define __ardour_editor_h__

#include <vector>
#include <list>

#include <gtk--.h>
#include <gtkwaveform-0.2/grange.h>

#include "seq.h"

namespace ARDOUR {
	class Tape;
	class Track;
}

namespace Gtkmmext {
	class Selector;
}

class AudioThing;

class Editor : public Gtk::EventBox
{
  public:
	Editor ();
	bool ok() { return _ok; }

	void connect_to_tape (ARDOUR::Tape *);

	void active ();
	void inactive ();

	int save_all_tracks ();

  protected:
	struct FakeTransportDisplay;
	friend struct FakeTransportDisplay;
	void map_transport_state ();

	void realize_impl();

  private:
	struct TrackView : public Gtk::VBox {
	    Editor  *editor;
	    ARDOUR::Track   *track;
	    int tracknum;

	    /* this lets us collect together different tracks
	       into edit groups. 
	    */

	    gint32 edit_group;

	    GtkWidget *waveview;
	    GtkObject *waveview_buffer;

	    void set_buffer (GtkObject *obj);

	    /* these are Gtk-- wrapped versions of the waveview widget
	       and the adjustment used by the waveview widget for 
	       x-axis position. they make our life as a programmer
	       a little easier.
	    */

	    Gtk::Widget *wave;            
	    Gtk::DrawingArea *wavearea;
	    Gtk::Adjustment *x_adjuster;

	    Gtk::Frame wave_frame;
	    Gtk::HBox  wave_hbox;
	    Gtk::HBox  hpacker;
	    Gtk::HBox  sep_box;
	    Gtk::HSeparator separator;
	    Gtk::HBox  button_hbox;
	    Gtk::VBox  left_vbox;
	    Gtk::Entry track_name_display;
	    Gtk::HBox  track_name_hbox;
	    Gtk::Frame track_name_frame;
	    Gtk::Entry take_name_display;
	    Gtk::HBox  take_name_hbox;
	    Gtk::Frame take_name_frame;

	    Gtk::ToggleButton mute_button;
	    Gtk::ToggleButton solo_button;
	    Gtk::Button       edit_button;
	    Gtk::Button       history_button;
	    Gtk::Label        edit_label;
	    
	    TrackView (Editor *, size_t tracknum, bool with_side_bar = true);
	    ~TrackView () {};
	    
	    void reset (ARDOUR::Tape *);
	    
	    /* History handling */

	    void history_click ();
	    gint history_hide (GdkEventAny *);

	    struct FakeHistoryDisplay : public Touchable {
		Gtkmmext::Selector *selector;
		void touch () {
			selector->rescan ();
		}
	    };

	    FakeHistoryDisplay fakeHistoryDisplay;

	    Gtkmmext::Selector   history_selector;
	    Gtk::Window          history_display;
	    static void _refill_history_list (Gtk::CList&, void *);
	    void refill_history_list (Gtk::CList&);
	    void random_history_selection (Gtkmmext::Selector *, Gtkmmext::SelectionResult *);
	    
	    static void _wave_modified (GtkObject *, GRange *, gpointer data);
	    void wave_modified (GRange *);
	};

	/* Since a trackview is a private class (an "implementation
	   aid") it has free access to everything here.
	*/

	friend struct TrackView;

	struct Region {
	    int source_track;
	    sequence_t *edl;
	    GtkObject *waveview_buffer;

	    string name;

	    Region(sequence_t *e, int track) { 
		    edl = e;
		    source_track = track;
	    }

	    ~Region () { 
		    sequence_free (edl);
	    }

	    void set_name (const string &str) {
		    name = str;
	    }
	};

	list<Region *> regions;
	Region        *cut_buffer; /* the most recently cut region */
	Region        *selected_region;

	Region *save_region (TrackView *, guint32 start, guint32 cnt);
	void    free_region (Region *);

	Gtk::Window        region_window;
	Gtk::HBox          region_hpacker;
	Gtk::VBox          region_left_vpacker;
	Gtk::VBox          region_right_vpacker;
	TrackView         *region_view;
	Gtk::Adjustment    region_xscroll_adjustment;
	Gtk::HScrollbar    region_xscroller;
	Gtk::HBox          region_xscroller_hbox;
	Gtk::VBox          region_wave_vbox;
 	Gtkmmext::Selector region_selector;

	static void  _region_refiller (Gtk::CList &, void *);
	void          region_refiller (Gtk::CList &);
	void          toggle_region_window ();
	void          build_region_window ();
	void          region_selected (Gtkmmext::Selector *, Gtkmmext::SelectionResult *);
	void          region_chosen (Gtkmmext::Selector *, Gtkmmext::SelectionResult *);
	void          region_rename_finished (bool status);
	bool          region_renamed;

	buffer_t *silent_buffer;

	/* enter/leave events set this, so that we
	   always know what, if any, is the current
	   trackview. well, sort of. we also set it
	   from button press and key press events
	   in the trackview drawing area.
	*/

	TrackView *current_trackview;
	Gtk::Menu *track_edit_menu;
	Gtk::Menu *region_edit_menu;
	Gtk::Menu *build_track_edit_menu ();
	
	ARDOUR::Tape   *tape;
	bool _ok;
	bool edits_legal;

	Gtk::HBox           hpacker;
	Gtk::VBox           vpacker;
	Gtk::VBox           view_packer;
	Gtk::HBox           bottom_box;
	Gtk::EventBox       channel_scroller_base;
	Gtk::ScrolledWindow channel_scroller;
	Gtk::HBox           xscroller_box;
	Gtk::Adjustment     xscroll_adjustment;
	Gtk::HScrollbar     xscroller;
	Gtk::Frame          cmd_indicator_frame;
	Gtk::HBox           cmd_indicator_box;
	Gtk::Entry          cmd_indicator;

	vector<TrackView *> track_views;
	vector<TrackView *> visible_track_views;
	vector<TrackView *> *get_valid_views (gint32 edit_group);

	Gtk::MenuBar      menu_bar;

	static GdkCursor *cross_hair_cursor;
	static void set_cross_hair_cursor (Gtk::DrawingArea *);

	gint scroll_timeout_tag;
	static gint _scroll_timeout (void *arg);
	gint scroll_timeout ();
	
	void build_menu_bar ();
	
	gint show_track_edit_menu (GdkEventButton *);
	void hide_track_edit_menu ();

	void tape_going_away (AudioThing *);

	guint32 cut_buffer_start;
	guint32 cut_buffer_length;

	void solo_toggle (gint32 track_num);
	void mute_toggle (gint32 track_num);
	gint edit_click  (GdkEventButton *, gint32 track_num);

	void xscroll_value_changed ();
	void region_xscroll_value_changed ();
	void reset_xscroll_steps (Gtk::Adjustment &, TrackView *);
	
	void tape_transport_state_change ();
	void reposition_cursor (size_t sample);
	void reposition_x_origin (size_t sample);

	static const guint32 max_edit_groups;

	gint button_press_handler (GdkEventButton *, TrackView *);
	gint button_release_handler (GdkEventButton *, TrackView *);

	/* KEYMAP HANDLING */

	bool escape_as_meta;
	bool prefix_pending;
	string prefix_string;
	bool control_c_pending;
	bool control_x_pending;

	gint key_press_handler (GdkEventKey *, TrackView *);

	gint do_plain_keymap (GdkEventKey *ev, 
			      bool control, bool shift,
			      bool meta, bool alt);
	gint do_control_keymap (GdkEventKey *ev, 
				bool control, bool shift,
				bool meta, bool alt);
	gint do_control_c_keymap (GdkEventKey *ev, 
				  bool control, bool shift,
				  bool meta, bool alt);
	gint do_control_x_keymap (GdkEventKey *ev, 
				  bool control, bool shift,
				  bool meta, bool alt);
	gint do_meta_keymap (GdkEventKey *ev, 
			     bool control, bool shift,
			     bool meta, bool alt);


	int ensure_cursor (gint32 *pos);
	int ensure_selection (gint32 *start, gint32 *cnt);

	/* EDITING OPERATIONS */

	void toggle_playback ();

	void temporal_zoom_step (bool in);
	void temporal_zoom_selection ();
	void temporal_zoom_tape ();
	void temporal_zoom (gdouble scale);
	void amplitude_zoom (gdouble scale);

	void cut_selection (bool actually_delete = true);
	void insert_cut_buffer ();
	void silence_selection ();
	void zero_sample_at_cursor ();

	void move_to_start ();
	void move_to_end ();
	void center_cursor ();
	void scroll_back_sample ();
	void scroll_forward_sample ();
	void scroll_back_screen ();
	void scroll_forward_screen ();
	void scroll_tracks_down ();
	void scroll_tracks_up ();
	void delete_sample_forward ();
	void delete_sample_backward ();
	void delete_screen ();
	void search_backwards ();
	void search_forwards ();
	void insert_silence ();
	void undo ();
	void redo ();
	void search ();
	void set_mark ();
	void jump_to_mark ();

	void reverse_selection ();
	void edit_envelope ();

	/* color selection */

	Gtk::ColorSelectionDialog color_dialog;
	void color_change_handler();
	void color_ok_handler ();
	void color_cancel_handler ();

	enum ColorType {
		NormalWave,
		SelectedWave,
		ActiveWave,
		ActiveBackground,
		NormalBackground,
		SelectedBackground
	};
	
	void set_color (ColorType);

	gdouble use_prefix (bool *was_floating);

	/* see comments in ardour_ui.h for an idea of how this works */

	struct FakeTransportDisplay : public Touchable {
	    Editor *editor;
	    void touch () {
		    editor->map_transport_state ();
	    }
	};
	  
	FakeTransportDisplay fakeTransportDisplay;

	bool restart_scrolling_when_active;
	void start_scrolling ();
	void stop_scrolling ();

	static void _cursor_set_handler (GtkObject *object, guint32 frame, gpointer arg) {
		((Editor *) arg)->cursor_set_handler (frame);
	}
	static void _selection_started_handler (GtkObject *object, guint32 frame, gpointer arg) {
		((Editor *) arg)->selection_started_handler (frame);
	}
	static void _selection_finished_handler (GtkObject *object, GRange *range, gpointer arg) {
		((Editor *) arg)->selection_finished_handler (range);
	}

	void cursor_set_handler (guint32 frame);
	void selection_started_handler (guint32 frame);
	void selection_finished_handler (GRange *frame);

	int  save_tracknum (int);
	void save_track ();
	void save_track_as ();
	
	/* Click tracks */

	Gtk::Window click_window;
	Gtk::Adjustment click_bpm_adjustment;
	Gtk::Adjustment click_bpb_adjustment;
	Gtk::Adjustment click_note_length_adjustment;
	Gtk::HScale click_bpm_scale;
	Gtk::SpinButton click_note_spinner;
	Gtk::SpinButton click_bpb_spinner;
	Gtk::VBox click_bpb_vbox;
	Gtk::VBox click_note_vbox;
	Gtk::VBox click_bpm_vbox;
	Gtk::Label click_bpm_label;
	Gtk::Label click_bpb_label;
	Gtk::Label click_note_label;
	Gtk::HBox click_upper_hpacker;
	Gtk::HBox click_lower_hpacker;
	Gtk::VBox click_vpacker;
	Gtk::Button click_ok_button;
	Gtk::Button click_cancel_button;

	void build_click_window ();
	void show_click_window (gint tracknum);
	void hide_click_window ();

	buffer_t *click_emphasis_sound;
	buffer_t *click_sound;
	piece_t  *click_emphasis_piece;
	piece_t  *click_piece;

	int build_click_buffers ();
	void print_click ();
	void do_click ();
};

#endif /* __ardour_editor_h__ */
