# Configure paths for Libgtkmmext
# Based on libsigc++ script by Karl Nelson

dnl Test for Libgtkmmext, and define GTKMMEXT_CFLAGS and GTKMMEXT_LIBS
dnl   to be used as follows:
dnl AM_PATH_GTKMMEXT(MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]])
dnl
AC_DEFUN(AM_PATH_GTKMMEXT,
[dnl 
dnl Get the cflags and libraries from the gtkmmext-config script
dnl

dnl
dnl Prefix options
dnl
AC_ARG_WITH(gtkmmext-prefix,
[  --with-gtkmmext-prefix=PREFIX
                          Prefix where libgtkmmext is installed (optional)]
, gtkmmext_config_prefix="$withval", gtkmmext_config_prefix="")

AC_ARG_WITH(gtkmmext-exec-prefix,
[  --with-gtkmmext-exec-prefix=PREFIX 
                          Exec prefix where  libgtkmmext is installed (optional)]
, gtkmmext_config_exec_prefix="$withval", gtkmmext_config_exec_prefix="")

AC_ARG_ENABLE(gtkmmexttest, 
[  --disable-gtkmmexttest     Do not try to compile and run a test libgtkmmext 
                          program],
, enable_gtkmmexttest=yes)

dnl
dnl Prefix handling
dnl
  if test x$gtkmmext_config_exec_prefix != x ; then
     gtkmmext_config_args="$gtkmmext_config_args --exec-prefix=$gtkmmext_config_exec_prefix"
     if test x${GTKMMEXT_CONFIG+set} != xset ; then
        GTKMMEXT_CONFIG=$gtkmmext_config_exec_prefix/bin/gtkmmext-config
     fi
  fi
  if test x$gtkmmext_config_prefix != x ; then
     gtkmmext_config_args="$gtkmmext_config_args --prefix=$gtkmmext_config_prefix"
     if test x${GTKMMEXT_CONFIG+set} != xset ; then
        GTKMMEXT_CONFIG=$gtkmmext_config_prefix/bin/gtkmmext-config
     fi
  fi

dnl
dnl See if gtkmmext-config is alive
dnl
  AC_PATH_PROG(GTKMMEXT_CONFIG, gtkmmext-config, no)
  gtkmmext_version_min=$1

dnl
dnl  Version check
dnl
  AC_MSG_CHECKING(for libgtkmmext - version >= $gtkmmext_version_min)
  no_gtkmmext=""
  if test "$GTKMMEXT_CONFIG" = "no" ; then
    no_gtkmmext=yes
  else
    gtkmmext_version=`$GTKMMEXT_CONFIG --version`

    GTKMMEXT_CFLAGS=`$GTKMMEXT_CONFIG $gtkmmext_config_args --cflags`
    GTKMMEXT_LIBS=`$GTKMMEXT_CONFIG $gtkmmext_config_args --libs`

    gtkmmext_major_version=`echo $gtkmmext_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    gtkmmext_minor_version=`echo $gtkmmext_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    gtkmmext_micro_version=`echo $gtkmmext_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`

    gtkmmext_major_min=`echo $gtkmmext_major_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    gtkmmext_minor_min=`echo $gtkmmext_minor_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    gtkmmext_micro_min=`echo $gtkmmext_micro_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`

    gtkmmext_version_proper=`expr \
        $gtkmmext_major_version \> $gtkmmext_major_min \| \
        $gtkmmext_major_version \= $gtkmmext_major_min \& \
        $gtkmmext_minor_version \> $gtkmmext_minor_min \| \
        $gtkmmext_major_version \= $gtkmmext_major_min \& \
        $gtkmmext_minor_version \= $gtkmmext_minor_min \& \
        $gtkmmext_micro_version \>= $gtkmmext_micro_min `

    if test "$gtkmmext_version_proper" = "1" ; then
      AC_MSG_RESULT([$gtkmmext_major_version.$gtkmmext_minor_version.$gtkmmext_micro_version])
    else
      AC_MSG_RESULT(no)
      no_gtkmmext=yes
    fi

    if test "X$no_gtkmmext" = "Xyes" ; then
      enable_gtkmmexttest=no
    fi

    AC_LANG_SAVE
    AC_LANG_CPLUSPLUS

dnl
dnl
dnl
    if test "x$enable_gtkmmexttest" = "xyes" ; then
      AC_MSG_CHECKING(if libgtkmmext sane)
      ac_save_CXXFLAGS="$CXXFLAGS"
      ac_save_LIBS="$LIBS"
      CXXFLAGS="$CXXFLAGS $GTKMMEXT_CFLAGS"
      LIBS="$LIBS $GTKMMEXT_LIBS"

      rm -f conf.gtkmmexttest

      AC_TRY_RUN([
#include <stdio.h>
#include <gtkmmext/version.h>
#include <pbd/transmitter.h>

Transmitter error(Transmitter::Error);
Transmitter info(Transmitter::Info);
Transmitter warning(Transmitter::Warning);
Transmitter fatal(Transmitter::Fatal);

int main(int argc,char **argv)
  {
   if (gtkmmext_major_version!=$gtkmmext_major_version ||
       gtkmmext_minor_version!=$gtkmmext_minor_version ||
       gtkmmext_micro_version!=$gtkmmext_micro_version)
     { printf("(%d.%d.%d) ",
         gtkmmext_major_version,gtkmmext_minor_version,gtkmmext_micro_version);
       return 1;
     }
  }

],[
  AC_MSG_RESULT(yes)
],[
  AC_MSG_RESULT(no)
  no_gtkmmext=yes
]
,[echo $ac_n "cross compiling; assumed OK... $ac_c"])

       CXXFLAGS="$ac_save_CXXFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi

  dnl
  dnl
  if test "x$no_gtkmmext" = x ; then
     ifelse([$2], , :, [$2])     
  else
     GTKMMEXT_CFLAGS=""
     GTKMMEXT_LIBS=""
     ifelse([$3], , :, [$3])
  fi

  AC_LANG_RESTORE

  AC_SUBST(GTKMMEXT_CFLAGS)
  AC_SUBST(GTKMMEXT_LIBS)
])

