;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  enumeration.scm
;;;  simple class to represent symbol to number mappings
;;;
;;;  Bill Gribble <grib@billgribble.com> 2/21/2000
;;;  $Id: enumeration.scm,v 1.1 2000/02/27 16:13:08 pbd Exp $
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(depend 'simple-obj)
(support 'enumeration)

;;  the "enumeration" class.  Just to make sure we can do reasonable
;;  type checking.  If you want to associate a specific value with 
;;  an enum tag, make that entry a cons cell.
;;
;;  examples: 
;;  (make-enumeration 'a 'b 'c 'd) ==>
;;    (enumeration . #(((a . 0) (b . 1) (c . 2) (d . 3))))
;;  
;;  (make-enumeration '(a . -5) 'b '(c . 4) 'd) ==> 
;;     (enumeration . #(((a . -5) (b . -4) (c . 4) (d . 5))))

(define <enumeration> 
  (make-simple-class 'enumeration
                     '(associations)))

(define (enumeration? self)
  (eq? (simple-obj-type self) 'enumeration))

(define (enumeration:associations self)
  (simple-obj-getter self <enumeration> 'associations))

(define (enumeration:set-associations! self value)
  (simple-obj-setter self <enumeration> 'associations value))

(define (enumeration:associate! self symbol value)
  (let ((old-alist (simple-obj-getter self <enumeration> 'associations)))    
    (simple-obj-setter self <enumeration> 
                       'associations (cons (cons symbol value) old-alist)))
  (cons symbol value))

(define (enumeration:symbol-to-value self value)
  (if (enumeration? self)
      (if (integer? value) ;; the case that we already have the value
          value 
          (let ((assoc-pair (assq value (enumeration:associations self))))
            (if assoc-pair
                (cdr assoc-pair)
                #f)))))

(define (enumeration:value-to-symbol self value)
  (if (enumeration? self)      
      (let ((assoc-pair 
             (let loop ((list-left (enumeration:associations self)))
               (if (null? list-left)
                   #f
                   (if (eq? (cdar list-left) value)
                       (car list-left)
                       (loop (cdr list-left)))))))
        (if (pair? assoc-pair)
            (car assoc-pair)
            assoc-pair))))
        
(define (make-enumeration . values)
  (let ((val (make-simple-obj <enumeration>))
        (count 0))
    (enumeration:set-associations! 
     val 
     (map (lambda (v) 
            (if (pair? v)
                (begin 
                  (set! count (cdr v))
                  (set! v (car v))))
            (let ((nv (cons v count)))
              (set! count (+ 1 count))
              nv))
          values))
    val))


