/*
    Copyright (C) 2000 Paul Barton-Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: memops.cc,v 1.9 2000/10/31 03:51:38 pbd Exp $
*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <memory.h>
#include <stdlib.h>

#include "memops.h"

/* XXX we could use rint(), but for now we'll accept whatever default
   floating-point => int conversion the compiler provides.  
*/

void sample_move_d32u24_sS (char *dst, ARDOUR::Sample *src, size_t nsamples, size_t dst_skip)

{
	/* ALERT: signed sign-extension portability !!! */

	while (nsamples--) {
		*((int *) dst) = ((int) (*src * SAMPLE_MAX_24BIT)) << 8;
		dst += dst_skip;
		src++;
	}
}	

void sample_move_dS_s32u24 (ARDOUR::Sample *dst, char *src, size_t nsamples, size_t src_skip)
{
	/* ALERT: signed sign-extension portability !!! */

	while (nsamples--) {
		*dst = (*((int *) src) >> 8) / SAMPLE_MAX_24BIT;
		dst++;
		src += src_skip;
	}
}	

void sample_move_d16_sS (char *dst,  ARDOUR::Sample *src, size_t nsamples, size_t dst_skip)
	
{
	/* ALERT: signed sign-extension portability !!! */
	
	while (nsamples--) {
		/* XXX good to use x86 assembler here, since float->short
		   sucks that h/w.
		*/
		   
		*((short *) dst) = (short) (*src * SAMPLE_MAX_16BIT);
		dst += dst_skip;
		src++;
	}
}	

void sample_move_dS_s16 (ARDOUR::Sample *dst, char *src, size_t nsamples, size_t src_skip) 
	
{
	/* ALERT: signed sign-extension portability !!! */
	while (nsamples--) {
		*dst = (*((short *) src)) / SAMPLE_MAX_16BIT;
		dst++;
		src += src_skip;
	}
}	

void sample_merge_d16_sS (char *dst,  ARDOUR::Sample *src, size_t nsamples, size_t dst_skip)
	
{
	/* ALERT: signed sign-extension portability !!! */
	
	while (nsamples--) {
		*((short *) dst) += (short) (*src * SAMPLE_MAX_16BIT);
		dst += dst_skip;
		src++;
	}
}	

void sample_merge_d32u24_sS (char *dst, ARDOUR::Sample *src, size_t nsamples, size_t dst_skip)

{
	/* ALERT: signed sign-extension portability !!! */

	while (nsamples--) {
		*((int *) dst) += (((int) (*src * SAMPLE_MAX_24BIT)) << 8);
		dst += dst_skip;
		src++;
	}
}	

void memset_interleave (char *dst, char val, unsigned int bytes, 
			unsigned int unit_bytes, 
			unsigned int skip_bytes) 
{
	switch (unit_bytes) {
	case 1:
		while (bytes--) {
			*dst = val;
			dst += skip_bytes;
		}
		break;
	case 2:
		while (bytes) {
			*((short *) dst) = (short) val;
			dst += skip_bytes;
			bytes -= 2;
		}
		break;
	case 4:		    
		while (bytes) {
			*((int *) dst) = (int) val;
			dst += skip_bytes;
			bytes -= 4;
		}
		break;
	}
}

/* COPY FUNCTIONS: used to move data from an input channel to an
   output channel. Note that we assume that the skip distance
   is the same for both channels. This is completely fine
   unless the input and output were on different audio interfaces that
   were interleaved differently. We don't try to handle that.
*/

void 
memcpy_fake (char *dst, char *src, size_t src_bytes, size_t foo)
{
	memcpy (dst, src, src_bytes);
}

void 
merge_memcpy_d16_s16 (char *dst, char *src, unsigned int src_bytes, size_t foo)
{
	while (src_bytes) {
		*((short *) dst) += *((short *) src);
		dst += 2;
		src += 2;
		src_bytes -= 2;
	}
}

void 
merge_memcpy_d32_s32 (char *dst, char *src, unsigned int src_bytes, size_t foo)
{
	while (src_bytes) {
		*((int *) dst) += *((int *) src);
		dst += 4;
		src += 4;
		src_bytes -= 4;
	}
}

void 
merge_memcpy_interleave_d16_s16 (char *dst, char *src, size_t src_bytes, size_t skip_bytes)
{
	while (src_bytes) {
		*((short *) dst) += *((short *) src);
		dst += skip_bytes;
		src += skip_bytes;
		src_bytes -= 2;
	}
}

void 
merge_memcpy_interleave_d32_s32 (char *dst, char *src, size_t src_bytes, size_t skip_bytes)
{
	while (src_bytes) {
		*((int *) dst) += *((int *) src);
		dst += skip_bytes;
		src += skip_bytes;
		src_bytes -= 4;
	}
}

void 
memcpy_interleave_d16_s16 (char *dst, char *src, size_t src_bytes, size_t skip_bytes)
{
	while (src_bytes) {
		*((short *) dst) = *((short *) src);
		dst += skip_bytes;
		src += skip_bytes;
		src_bytes -= 2;
	}
}

void 
memcpy_interleave_d32_s32 (char *dst, char *src, size_t src_bytes, size_t skip_bytes)

{
	while (src_bytes) {
		*((int *) dst) = *((int *) src);
		dst += skip_bytes;
		src += skip_bytes;
		src_bytes -= 4;
	}
}


