/*
    Copyright (C) 1999 Paul Barton-Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: track_ui.h,v 1.21 2000/10/14 14:14:35 pbd Exp $
*/

#ifndef __ardour_track_ui_h__
#define __ardour_track_ui_h__

#include <sigc++/signal_system.h>
#include <pbd/touchable.h>
#include <gtk--.h>
#include <gtkmmext/pixmap_button.h>
#include <gtkmmext/levelhold.h>
#include <gtkmmext/led.h>
#include <gtkmmext/click_box.h>

namespace ALSA {
	class MultiChannelDevice;
}

namespace ARDOUR {
	class Tape;
	class Track;
}

namespace Gtkmmext {
	class Selector;
	struct SelectionResult;
}

class AudioThing;

class Track_UI : public SigC::Object, public Touchable

{
  public:
	Track_UI (ALSA::MultiChannelDevice *, size_t track_num);
	
	void update ();  /* called by meter timeout handler from ARDOUR_UI */
	void touch ();   /* called in response to inter-thread updates */

	Gtk::Box &above_box() { return above_meter_vbox; }
	Gtk::Box &below_box() { return below_meter_vbox; }
	Gtk::Box &meter_box() { return meter_hbox; }
	
	size_t meter_width() { return 8; }

	void connect_to_tape (ARDOUR::Tape *);

	void clear_meter ();
	void clear_overs ();

	void record_enable_blink (bool onoff);
	void allow_record_enable ();
	void disallow_record_enable ();

	void show_lock_and_sync () { 
		input_ok_led.green();
	}
	void show_lock () { 
		input_ok_led.green(); 
		input_ok_led.blink(); 
	}
	void show_no_lock () { 
		input_ok_led.black(); 
	}

  private:
	ALSA::MultiChannelDevice *mcd;
	ARDOUR::Tape           *tape;
	ARDOUR::Track          *track;
	size_t                  channel_num;
	Gtk::Label              label;
	Gtk::HBox               rec_enable_hbox;
	Gtk::HBox               monitor_hbox;
	Gtkmmext::PixmapButton  rec_enable;
	Gtkmmext::PixmapButton  monitor_toggle;
	bool                    monitor_toggle_pending;
	bool                    meter_clear_pending;
	bool                    over_clear_pending;

	Gtk::LevelHold     meter;
	Gtk::HBox          meter_hbox;

	Gtk::VBox          above_meter_vbox;
	Gtk::VBox          below_meter_vbox;

	Gtk::HBox                take_hbox;
	Gtkmmext::PixmapButton   take_button;
	
	Gtk::Adjustment    inchan_adjustment;
	Gtkmmext::ClickBox inchan;
	Gtk::HBox          inchan_hbox;
	Gtk::Frame         inchan_frame;

	Gtk::Adjustment    outchan_adjustment;
	Gtkmmext::ClickBox outchan;
	Gtk::HBox          outchan_hbox;
	Gtk::Frame         outchan_frame;

	Gtk::HBox          over_long_hbox;
	Gtk::HBox          over_long_vbox;
	Gtk::EventBox      over_long_button;
	Gtk::Frame         over_long_frame;
	Gtk::Label         over_long_label;

	Gtk::HBox          over_short_hbox;
	Gtk::HBox          over_short_vbox;
	Gtk::EventBox      over_short_button;
	Gtk::Frame         over_short_frame;
	Gtk::Label         over_short_label;

	Gtkmmext::LED      input_ok_led;
	Gtk::HBox          input_ok_led_box;
	
	size_t            last_over_short;
	size_t            last_over_long;

	static Gtk::Window   *take_window;
	static Gtk::Tooltips *track_tooltips;
	static Gtk::Entry    *take_name_display;
	static Gtk::Entry    *track_name_display;

	gint toggle_rec (GdkEventButton *);
	gint toggle_monitor (GdkEventButton *);
	void disconnect_from_tape (AudioThing *);

	void inchan_adjusted ();
	void outchan_adjusted ();
	void take_clicked ();
	gint gui_clear_overs (GdkEventButton *);

	void take_or_tag_changed ();
	void input_monitor_change (size_t chn, bool yn);
	void track_tag_display_changed ();
	void take_tag_display_changed ();
	void map_transport_state ();

	void new_take ();
	void erase_take ();
	void select_take (size_t);
	void take_cancelled ();
	static void take_selected (Gtkmmext::Selector *, Gtkmmext::SelectionResult *);
	
	void create_tooltips ();
	void create_menus_and_dialogs ();

	static void take_lister (Gtk::CList &list, void *arg);
	static bool take_filter (const string &str);
};

#endif  /* __ardour_track_ui_h__ */
